package com.sun.xml.internal.messaging.saaj.soap;

import javax.xml.transform.Transformer;
import java.io.StringReader;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamResult;
import com.sun.xml.internal.messaging.saaj.util.transform.EfficientStreamingTransformer;
import java.io.OutputStream;
import java.io.IOException;
import javax.xml.transform.stream.StreamSource;
import javax.activation.DataSource;
import javax.activation.ActivationDataFlavor;
import java.awt.datatransfer.DataFlavor;
import javax.activation.DataContentHandler;

public class XmlDataContentHandler implements DataContentHandler
{
    public static final String STR_SRC = "javax.xml.transform.stream.StreamSource";
    private static Class streamSourceClass;
    
    public XmlDataContentHandler() throws ClassNotFoundException {
        if (XmlDataContentHandler.streamSourceClass == null) {
            XmlDataContentHandler.streamSourceClass = Class.forName("javax.xml.transform.stream.StreamSource");
        }
    }
    
    public DataFlavor[] getTransferDataFlavors() {
        DataFlavor[] flavors = { new ActivationDataFlavor(XmlDataContentHandler.streamSourceClass, "text/xml", "XML"), new ActivationDataFlavor(XmlDataContentHandler.streamSourceClass, "application/xml", "XML") };
        return flavors;
    }
    
    public Object getTransferData(DataFlavor flavor, DataSource dataSource) throws IOException {
        if ((flavor.getMimeType().startsWith("text/xml") || flavor.getMimeType().startsWith("application/xml")) && flavor.getRepresentationClass().getName().equals("javax.xml.transform.stream.StreamSource")) {
            return new StreamSource(dataSource.getInputStream());
        }
        return null;
    }
    
    public Object getContent(DataSource dataSource) throws IOException {
        return new StreamSource(dataSource.getInputStream());
    }
    
    public void writeTo(Object obj, String mimeType, OutputStream os) throws IOException {
        if (!mimeType.startsWith("text/xml") && !mimeType.startsWith("application/xml")) {
            throw new IOException("Invalid content type \"" + mimeType + "\" for XmlDCH");
        }
        try {
            Transformer transformer = EfficientStreamingTransformer.newTransformer();
            StreamResult result = new StreamResult(os);
            if ((obj instanceof DataSource)) {
                transformer.transform((Source)this.getContent((DataSource)obj), result);
            }
            else {
                Source src = null;
                if ((obj instanceof String)) {
                    src = new StreamSource(new StringReader((String)obj));
                }
                else {
                    src = (Source)obj;
                }
                transformer.transform(src, result);
            }
        }
        catch (Exception ex) {
            throw new IOException("Unable to run the JAXP transformer on a stream " + ex.getMessage());
        }
        return;
    }
    
    static {
        XmlDataContentHandler.streamSourceClass = null;
    }
}
